@@ -30,6 +30,7 @@
 
 import math
 import numbers
+import os
 
 import awkward.array.base
 import awkward.persist
@@ -193,6 +194,14 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         jagged = jagged._tojagged(copy=False)
         return cls(jagged._starts, jagged._stops, jagged._content)
 
+    @classmethod
+    def fromregular(cls, content, size=1):
+        quotient = -(-len(content) // size)
+        offsets = awkward.util.numpy.arange(0, quotient * size + 1, size, dtype=awkward.util.INDEXTYPE)
+        if len(offsets) > 0:
+            offsets[-1] = len(content)
+        return cls.fromoffsets(offsets, content)
+
     def copy(self, starts=None, stops=None, content=None):
         out = self.__class__.__new__(self.__class__)
         out._starts  = self._starts
@@ -461,7 +470,7 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
                     thyself._stops.shape = self._stops.shape
 
                 head = head._tojagged(thyself._starts, thyself._stops, copy=False)
-                inthead = head.copy(content=head._content.view(awkward.util.numpy.uint8))
+                inthead = head.copy(content=head._content.astype(awkward.util.INDEXTYPE))
                 intheadsum = inthead.sum()
 
                 offsets = counts2offsets(intheadsum)
@@ -493,6 +502,74 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
                 if not awkward.util.numpy.bitwise_and(0 <= head, head < counts).all():
                     raise IndexError("index {0} is out of bounds for jagged min size {1}".format(original_head, counts.min()))
                 node = node._content[node._starts + head]
+
+            elif isinstance(head, slice):
+                counts = node._stops - node._starts
+                step = 1 if head.step is None else head.step
+
+                if step == 0:
+                    raise ValueError("slice step cannot be zero")
+
+                elif step > 0:
+                    if head.start is None:
+                        starts = awkward.util.numpy.zeros(counts.shape, dtype=awkward.util.INDEXTYPE)
+                    elif head.start >= 0:
+                        starts = awkward.util.numpy.minimum(counts, head.start)
+                    else:
+                        starts = awkward.util.numpy.maximum(0, awkward.util.numpy.minimum(counts, counts + head.start))
+
+                    if head.stop is None:
+                        stops = counts
+                    elif head.stop >= 0:
+                        stops = awkward.util.numpy.minimum(counts, head.stop)
+                    else:
+                        stops = awkward.util.numpy.maximum(0, awkward.util.numpy.minimum(counts, counts + head.stop))
+
+                    stops = awkward.util.numpy.maximum(starts, stops)
+
+                    start = starts.min()
+                    stop = stops.max()
+                    indexes = awkward.util.numpy.empty((len(node), abs(stop - start)), dtype=awkward.util.INDEXTYPE)
+                    indexes[:, :] = awkward.util.numpy.arange(start, stop)
+
+                    mask = indexes >= starts.reshape((len(node), 1))
+                    awkward.util.numpy.bitwise_and(mask, indexes < stops.reshape((len(node), 1)), out=mask)
+                    if step != 1:
+                        awkward.util.numpy.bitwise_and(mask, awkward.util.numpy.remainder(indexes - starts.reshape((len(node), 1)), step) == 0, out=mask)
+
+                else:
+                    if head.start is None:
+                        starts = counts - 1
+                    elif head.start >= 0:
+                        starts = awkward.util.numpy.minimum(counts - 1, head.start)
+                    else:
+                        starts = awkward.util.numpy.maximum(-1, awkward.util.numpy.minimum(counts - 1, counts + head.start))
+                    
+                    if head.stop is None:
+                        stops = awkward.util.numpy.full(counts.shape, -1, dtype=awkward.util.INDEXTYPE)
+                    elif head.stop >= 0:
+                        stops = awkward.util.numpy.minimum(counts - 1, head.stop)
+                    else:
+                        stops = awkward.util.numpy.maximum(-1, awkward.util.numpy.minimum(counts - 1, counts + head.stop))
+
+                    stops = awkward.util.numpy.minimum(starts, stops)
+
+                    start = starts.max()
+                    stop = stops.min()
+                    indexes = awkward.util.numpy.empty((len(node), abs(stop - start)), dtype=awkward.util.INDEXTYPE)
+                    indexes[:, :] = awkward.util.numpy.arange(start, stop, -1)
+
+                    mask = indexes <= starts.reshape((len(node), 1))
+                    awkward.util.numpy.bitwise_and(mask, indexes > stops.reshape((len(node), 1)), out=mask)
+                    if step != -1:
+                        awkward.util.numpy.bitwise_and(mask, awkward.util.numpy.remainder(indexes - starts.reshape((len(node), 1)), step) == 0, out=mask)
+
+                newcounts = awkward.util.numpy.count_nonzero(mask, axis=1)
+                newoffsets = counts2offsets(newcounts.reshape(-1))
+                newcontent = node.content[(indexes + node._starts.reshape((len(node), 1)))[mask]]
+
+                node = node.copy(starts=newoffsets[:-1], stops=newoffsets[1:], content=newcontent)
+
             else:
                 # the other cases are possible, but complicated; the first sets the form
                 raise NotImplementedError("jagged second dimension index type: {0}".format(original_head))
@@ -677,6 +754,20 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         else:
             return awkward.array.objects.Methods.maybemixin(type(result), JaggedArray).fromcounts(counts, result)
 
+    def regular(self):
+        if len(self) > 0 and not (self.counts.reshape(-1)[0] == self.counts).all():
+            raise ValueError("jagged array is not regular: different elements have different counts")
+        count = self.counts.reshape(-1)[0]
+        
+        if self._canuseoffset():
+            out = self._content[self._starts[0]:self._stops[-1]]
+            return out.reshape(self._starts.shape + (count,) + self._content.shape[1:])
+
+        else:
+            indexes = awkward.util.numpy.repeat(self._starts, count).reshape(self._starts.shape + (count,))
+            indexes += awkward.util.numpy.arange(count)
+            return self._content[indexes]
+
     @staticmethod
     def aligned(*jaggedarrays):
         if not all(isinstance(x, JaggedArray) for x in jaggedarrays):
@@ -708,6 +799,12 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
         return True
 
+    def argdistincts(self):
+        return self.argpairs(same=False)
+
+    def distincts(self):
+        return self.pairs(same=False)
+
     def argpairs(self, same=True):
         import awkward.array.table
         self._valid()
@@ -810,6 +907,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
             out = awkward.util.numpy.empty(self._starts.shape + content.shape[1:], dtype=content.dtype)
             nonterminal = self.offsets[self.offsets != self.offsets[-1]]
+            if os.name == "nt":   # Windows Numpy reduceat requires 32-bit indexes
+                nonterminal = nonterminal.astype(awkward.util.numpy.int32)
             out[:len(nonterminal)] = awkward.util.numpy.logical_or.reduceat(content[self._starts[0]:self._stops[-1]], nonterminal)
             out[self.offsets[1:] == self.offsets[:-1]] = False
             return out
@@ -826,6 +925,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
             out = awkward.util.numpy.empty(self._starts.shape + content.shape[1:], dtype=content.dtype)
             nonterminal = self.offsets[self.offsets != self.offsets[-1]]
+            if os.name == "nt":   # Windows Numpy reduceat requires 32-bit indexes
+                nonterminal = nonterminal.astype(awkward.util.numpy.int32)
             out[:len(nonterminal)] = awkward.util.numpy.logical_and.reduceat(content[self._starts[0]:self._stops[-1]], nonterminal)
             out[self.offsets[1:] == self.offsets[:-1]] = True
             return out
@@ -848,6 +949,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         if self._canuseoffset():
             out = awkward.util.numpy.empty(self._starts.shape + content.shape[1:], dtype=content.dtype)
             nonterminal = self.offsets[self.offsets != self.offsets[-1]]
+            if os.name == "nt":   # Windows Numpy reduceat requires 32-bit indexes
+                nonterminal = nonterminal.astype(awkward.util.numpy.int32)
             out[:len(nonterminal)] = awkward.util.numpy.add.reduceat(content[self._starts[0]:self._stops[-1]], nonterminal)
             out[self.offsets[1:] == self.offsets[:-1]] = 0
             return out
@@ -872,6 +975,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         if self._canuseoffset():
             out = awkward.util.numpy.empty(self._starts.shape + content.shape[1:], dtype=content.dtype)
             nonterminal = self.offsets[self.offsets != self.offsets[-1]]
+            if os.name == "nt":   # Windows Numpy reduceat requires 32-bit indexes
+                nonterminal = nonterminal.astype(awkward.util.numpy.int32)
             out[:len(nonterminal)] = awkward.util.numpy.multiply.reduceat(content[self._starts[0]:self._stops[-1]], nonterminal)
             out[self.offsets[1:] == self.offsets[:-1]] = 1
             return out
@@ -931,6 +1036,8 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
     def _minmax_offset(self, ismin):
         out = awkward.util.numpy.empty(self._starts.shape + self._content.shape[1:], dtype=self._content.dtype)
         nonterminal = self.offsets[self.offsets != self.offsets[-1]]
+        if os.name == "nt":   # Windows Numpy reduceat requires 32-bit indexes
+            nonterminal = nonterminal.astype(awkward.util.numpy.int32)
 
         if ismin:
             out[:len(nonterminal)] = awkward.util.numpy.minimum.reduceat(self._content[self._starts[0]:self._stops[-1]], nonterminal)
@@ -1013,23 +1120,22 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         else:
             return self._minmax_general(False, False)
 
-    @classmethod
-    def regular(cls, content, size=1):
-        quotient = -(-len(content) // size)
-        offsets = awkward.util.numpy.arange(0, quotient * size + 1, size, dtype=awkward.util.INDEXTYPE)
-        if len(offsets) > 0:
-            offsets[-1] = len(content)
-        return cls.fromoffsets(offsets, content)
+    @awkward.util.bothmethod
+    def concatenate(isclassmethod, cls_or_self, arrays):
+        if isclassmethod: 
+            cls = cls_or_self
+            if not all(isinstance(x, JaggedArray) for x in arrays):
+                raise TypeError("cannot concatenate non-JaggedArrays with JaggedArray.concatenate")
+        else:
+            self = cls_or_self
+            cls = self.__class__
+            if not isinstance(self, JaggedArray) or not all(isinstance(x, JaggedArray) for x in arrays):
+                raise TypeError("cannot concatenate non-JaggedArrays with JaggedArray.concatenate")
+            arrays = (self,) + tuple(arrays)
 
-    @classmethod
-    def concat(cls, first, *rest):    # all elements of first followed by all elements of second
-        arrays = (first,) + rest
         for x in arrays:
             x._valid()
 
-        if not all(isinstance(x, JaggedArray) for x in arrays):
-            raise TypeError("cannot concat JaggedArrays with non-JaggedArrays")
-
         starts = awkward.util.numpy.concatenate([x._starts for x in arrays])
         stops = awkward.util.numpy.concatenate([x._stops for x in arrays])
         content = awkward.util.concatenate([x._content for x in arrays])
@@ -1133,6 +1239,14 @@ class ByteJaggedArray(JaggedArray):
         tmp = ByteJaggedArray.__bases__[0].fromuniques(uniques, awkward.util.numpy.array([]))
         return cls(tmp._starts, tmp._stops, content, subdtype=subdtype)
 
+    @classmethod
+    def fromregular(cls, content, size=1):
+        quotient = -(-len(content) // size)
+        offsets = awkward.util.numpy.arange(0, quotient * size + 1, size, dtype=awkward.util.INDEXTYPE)
+        if len(offsets) > 0:
+            offsets[-1] = len(content)
+        return cls.fromoffsets(offsets, content)
+
     def copy(self, starts=None, stops=None, content=None, subdtype=None):
         out = super(ByteJaggedArray, self).copy(starts=starts, stops=stops, content=content)
         if subdtype is None:
